@@ -76,9 +76,9 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.11',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/build/python3.11-d8xZ0N/python3.11-3.11.4/build-shared/coverage.info',
+ 'COVERAGE_INFO': '/build/python3.11-k8cXm3/python3.11-3.11.6/build-shared/coverage.info',
  'COVERAGE_LCOV_OPTIONS': '--rc lcov_branch_coverage=1',
- 'COVERAGE_REPORT': '/build/python3.11-d8xZ0N/python3.11-3.11.4/build-shared/lcov-report',
+ 'COVERAGE_REPORT': '/build/python3.11-k8cXm3/python3.11-3.11.6/build-shared/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--rc lcov_branch_coverage=1 --branch-coverage '
                             '--title "CPython 3.11 LCOV report [commit $(shell '
                             ')]"',
@@ -88,6 +88,7 @@ build_time_vars = {'ABIFLAGS': '',
  'DECIMAL_CFLAGS': '-I../Modules/_decimal/libmpdec -DCONFIG_64=1 -DANSI=1 '
                    '-DHAVE_UINT128_T=1',
  'DECIMAL_LDFLAGS': '-lm Modules/_decimal/libmpdec/libmpdec.a',
+ 'DEEPFREEZE_C': 'Python/deepfreeze/deepfreeze.c',
  'DEEPFREEZE_DEPS': '../Tools/scripts/deepfreeze.py _bootstrap_python '
                     '../Programs/_freeze_module.py \\',
  'DEEPFREEZE_OBJS': 'Python/deepfreeze/deepfreeze.o',
@@ -354,6 +355,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_LINUX_CAN_RAW_FD_FRAMES': 1,
  'HAVE_LINUX_CAN_RAW_H': 1,
  'HAVE_LINUX_CAN_RAW_JOIN_FILTERS': 1,
+ 'HAVE_LINUX_LIMITS_H': 1,
  'HAVE_LINUX_MEMFD_H': 1,
  'HAVE_LINUX_NETLINK_H': 1,
  'HAVE_LINUX_QRTR_H': 1,
@@ -510,7 +512,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_STRFTIME': 1,
  'HAVE_STRINGS_H': 1,
  'HAVE_STRING_H': 1,
- 'HAVE_STRLCPY': 0,
+ 'HAVE_STRLCPY': 1,
  'HAVE_STROPTS_H': 0,
  'HAVE_STRSIGNAL': 1,
  'HAVE_STRUCT_PASSWD_PW_GECOS': 1,
@@ -1102,7 +1104,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python3.11-d8xZ0N/python3.11-3.11.4/build-shared',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python3.11-k8cXm3/python3.11-3.11.6/build-shared',
  'SCRIPTDIR': '/usr/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SHAREDMODS': 'Modules/_asyncio.cpython-311-powerpc64le-linux-gnu.so '
@@ -1189,10 +1191,10 @@ build_time_vars = {'ABIFLAGS': '',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python3.11-d8xZ0N/python3.11-3.11.4/build-shared '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python3.11-k8cXm3/python3.11-3.11.6/build-shared '
                './python -E',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python3.11-d8xZ0N/python3.11-3.11.4/build-shared '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python3.11-k8cXm3/python3.11-3.11.6/build-shared '
                './python -E ../Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'ctypes/test \\',
  'TESTTIMEOUT': 1200,
@@ -1224,8 +1226,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/build/python3.11-d8xZ0N/python3.11-3.11.4/build-shared',
- 'abs_srcdir': '/build/python3.11-d8xZ0N/python3.11-3.11.4/build-shared/..',
+ 'abs_builddir': '/build/python3.11-k8cXm3/python3.11-3.11.6/build-shared',
+ 'abs_srcdir': '/build/python3.11-k8cXm3/python3.11-3.11.6/build-shared/..',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
